﻿using System;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Xml;
using NFeLib.NFE.BaseNF.Servicos.Autorizacao;
using NFeLib.NFE.BaseNF.Servicos.ConsSitNFe;
using NFeLib.NFE.BaseNF.Servicos.ConsultaCadastro;
using NFeLib.NFE.BaseNF.Servicos.DistribuicaoDFe;
using NFeLib.NFE.BaseNF.Servicos.DownloadNFe;
using NFeLib.NFE.BaseNF.Servicos.Inutilizacao;
using NFeLib.NFE.BaseNF.Servicos.RecepcaoEvento;
using NFeLib.NFE.BaseNF.Servicos.Status;
using NFeLib.NFE.Utils;

namespace NFeLib.NFE.WebServices
{
    public class WebRequestWS : IWebService
    {
        private ConfigWsUrls _nfeConfig;
        private X509Certificate2 _certificadoDigital;
        private int _timeout = 30000;
        private string _erros = string.Empty;
        private string _nomeClasse = string.Empty;

        public WebRequestWS(ConfigWsUrls nfeConfig, X509Certificate2 certDigital, string nomeClasse)
        {
            this._nfeConfig = nfeConfig;
            _certificadoDigital = certDigital;
            _nomeClasse = nomeClasse;
        }

        #region - Métodos Públicos

        public int Timeout
        {
            set
            {
                _timeout = value;
            }
        }

        public string GetErros
        {
            get
            {
                return _erros;
            }
        }

        public retEnviNFe NFeAutorizacao(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retEnviNFe>(xmlDoc);
        }

        public retConsReciNFe NFeRetAutorizacao(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retConsReciNFe>(xmlDoc);
        }

        public retConsSitNFe NFeConsultaProtocolo(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retConsSitNFe>(xmlDoc);
        }

        public retEnvEvento RecepcaoEvento(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retEnvEvento>(xmlDoc);
        }

        public retConsStatServ NFeStatusServico(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retConsStatServ>(xmlDoc);
        }

        public retInutNFe NFeInutilizacao(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retInutNFe>(xmlDoc);
        }

        public retDownloadNFe NFeDownloadNF(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retDownloadNFe>(xmlDoc);
        }

        public retDistDFeInt NFeDistribuicaoDFe(XmlDocument xmlDoc)
        {
            XmlDocument xmlSoapEnvio = SoapNFeDistribuicaoDFe(xmlDoc);
            return EnviarSefaz<retDistDFeInt>(xmlSoapEnvio, true, false, false);
        }

        public retConsCad NFeConsultaCadastro(XmlDocument xmlDoc)
        {
            return EnviarSefaz<retConsCad>(xmlDoc);
        }

        public XmlDocument XmlRetornoSefaz {get; set; }
       
        #endregion

        #region - Métodos Privados

        private T EnviarSefaz<T>(XmlDocument xmlDoc, bool deserializar = true, bool cabecMsg = true,bool criarSoap = true) where T : class, new()
        {
            XmlRetornoSefaz = null;
            XmlDocument xmlSoapEnvio = new XmlDocument();
            xmlSoapEnvio = (criarSoap) ? SoapXmlNFe(xmlDoc) : xmlDoc;

            XmlDocument xmlRetorno = EnviarSefaz(xmlSoapEnvio);
            XmlDocument xmlSerializar = new XmlDocument();
            if (xmlRetorno == null)
            {
                return null;
            }

            xmlSerializar = ExtractNodesXmlSoap(xmlRetorno, new T().GetType().Name);
            XmlRetornoSefaz = xmlSerializar;
    
            if (deserializar)
            {
                return FunctionNFe.DeserializarNFe<T>(xmlSerializar.OuterXml);
            }

            object obj = xmlSerializar;
            return (T)obj;

        }

        private XmlDocument EnviarSefaz(XmlDocument xmlDoc)
        {
            try
            {
                // Configuração de protocolo TLS .Net Framework 4.5
                ServicePointManager.ServerCertificateValidationCallback = delegate { return true; };
                ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12 | SecurityProtocolType.Ssl3;

                // Configuração de protocolo TLS .Net Framework 4.0
                //ServicePointManager.SecurityProtocol = (SecurityProtocolType)48 | (SecurityProtocolType)192 | (SecurityProtocolType)768 | (SecurityProtocolType)3072;

                XmlDocument xmlRetorno = new XmlDocument();
                HttpWebRequest httpWebNFe = (HttpWebRequest)WebRequest.Create(_nfeConfig.Url);
                byte[] byteArray = Encoding.UTF8.GetBytes(xmlDoc.InnerXml);

                httpWebNFe.PreAuthenticate = true;
                httpWebNFe.ServicePoint.Expect100Continue = false;
                httpWebNFe.CookieContainer = new CookieContainer();
                httpWebNFe.Timeout = _timeout;
                httpWebNFe.Headers.Add("SOAP:Action");
                httpWebNFe.ContentType = "application/soap+xml;charset=utf-8";
                httpWebNFe.Method = WebRequestMethods.Http.Post;
                httpWebNFe.Accept = "text/xml";
                httpWebNFe.KeepAlive = false;
                httpWebNFe.ClientCertificates.Add(_certificadoDigital);
                httpWebNFe.ContentLength = byteArray.Length;
                httpWebNFe.Proxy = WebRequest.GetSystemWebProxy();

                Stream dataStream = httpWebNFe.GetRequestStream();
                dataStream.Write(byteArray, 0, byteArray.Length);

                WebResponse response = httpWebNFe.GetResponse();
                dataStream = response.GetResponseStream();
                xmlRetorno.Load(dataStream);

                dataStream.Close();
                response.Close();

                return xmlRetorno;

            }

            catch (WebException ex)
            {
                _erros = ex.Message;
                return null;
            }
        }

        private XmlDocument SoapXmlNFe(XmlDocument xmlLotesNFe)
        {
            try
            {
                if (xmlLotesNFe.FirstChild is XmlDeclaration)
                {
                    xmlLotesNFe.RemoveChild(xmlLotesNFe.FirstChild);
                }

                string nsRoot = "http://www.w3.org/2003/05/soap-envelope";
                string nsUrlNFe = "http://www.portalfiscal.inf.br/nfe/wsdl/" + _nomeClasse;
                XmlDocument xmlDocument = new XmlDocument();

                xmlDocument.AppendChild(xmlDocument.CreateXmlDeclaration("1.0", "UTF-8", null));
                XmlElement envelope = xmlDocument.CreateElement("soap12:Envelope", nsRoot);

                envelope.SetAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
                envelope.SetAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");

                XmlElement body = xmlDocument.CreateElement("soap12", "Body", nsRoot);
                XmlElement nfeDadosMsg = xmlDocument.CreateElement("nfeDadosMsg", nsUrlNFe);
                nfeDadosMsg.InnerXml = xmlLotesNFe.InnerXml;
                body.AppendChild(nfeDadosMsg);
                envelope.AppendChild(body);

                xmlDocument.AppendChild(envelope);
                return xmlDocument;
            }

            catch (Exception ex)
            {
                _erros = ex.Message;
                return null;
            }
        }

        private XmlDocument ExtractNodesXmlSoap(XmlDocument xmlDoc, string tag)
        {
            try
            {
                XmlNodeList xmlList = xmlDoc.GetElementsByTagName(tag);
                XmlDocument xmlRetorno = new XmlDocument();
                xmlRetorno.LoadXml(xmlList[0].OuterXml);
                return xmlRetorno;
            }

            catch (Exception ex)
            {
                _erros = ex.Message;
                return null;
            }
        }

        private XmlDocument SoapNFeDistribuicaoDFe(XmlDocument xmlLotesNFe)
        {
            try
            {
                if (xmlLotesNFe.FirstChild is XmlDeclaration)
                {
                    xmlLotesNFe.RemoveChild(xmlLotesNFe.FirstChild);
                }

                string nsRoot = "http://www.w3.org/2003/05/soap-envelope";
                XmlDocument doc = new XmlDocument();

                doc.AppendChild(doc.CreateXmlDeclaration("1.0", "UTF-8", null));
                XmlElement envelope = doc.CreateElement("soap12:Envelope", nsRoot);

                envelope.SetAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
                envelope.SetAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");

                XmlElement body = doc.CreateElement("soap12", "Body", nsRoot);
                XmlElement nfeDistDFeInteresse = doc.CreateElement("nfeDistDFeInteresse", "http://www.portalfiscal.inf.br/nfe/wsdl/NFeDistribuicaoDFe");

                XmlElement nfeDadosMsg = doc.CreateElement("nfeDadosMsg", "http://www.portalfiscal.inf.br/nfe/wsdl/NFeDistribuicaoDFe");
                nfeDadosMsg.InnerXml = xmlLotesNFe.InnerXml;

                nfeDistDFeInteresse.AppendChild(nfeDadosMsg);
                body.AppendChild(nfeDistDFeInteresse);
                envelope.AppendChild(body);

                doc.AppendChild(envelope);
                return doc;

            }

            catch (Exception ex)
            {
                _erros = ex.Message;
                return null;
            }
        }
        #endregion
    }
}
